@@ -202,7 +202,7 @@ class CollectionVersionSignatureSerializer(serializers.ModelSerializer):
         """
         Get the signature data.
         """
-        return bytes(obj.data).decode("utf-8")
+        return obj.data
 
     class Meta:
         model = models.CollectionVersionSignature
@@ -327,3 +327,9 @@ class RepoMetadataSerializer(serializers.ModelSerializer):
     class Meta:
         fields = ("published",)
         model = RepositoryVersion
+
+
+class ClientConfigurationSerializer(serializers.Serializer):
+    """Configuration settings for the ansible-galaxy client."""
+
+    default_distribution_path = serializers.CharField(allow_null=True)
