@@ -33,6 +33,7 @@ from pulpcore.plugin.models import (
     PulpTemporaryFile,
     Remote,
     RepositoryContent,
+    RepositoryVersion,
     Task,
 )
 from pulpcore.plugin.stages import (
@@ -62,7 +63,7 @@ from pulp_ansible.app.models import (
     CollectionVersionSignature,
     Tag,
 )
-from pulp_ansible.app.serializers import CollectionRemoteSerializer, CollectionVersionSerializer
+from pulp_ansible.app.serializers import CollectionVersionSerializer
 from pulp_ansible.app.tasks.utils import (
     RequirementsFileEntry,
     get_file_obj_from_tarball,
@@ -77,12 +78,14 @@ async def declarative_content_from_git_repo(remote, url, git_ref=None, metadata_
     """Returns a DeclarativeContent for the Collection in a Git repository."""
     if git_ref:
         try:
-            gitrepo = Repo.clone_from(url, uuid4(), depth=1, branch=git_ref)
+            gitrepo = Repo.clone_from(
+                url, uuid4(), depth=1, branch=git_ref, multi_options=["--recurse-submodules"]
+            )
         except GitCommandError:
-            gitrepo = Repo.clone_from(url, uuid4())
+            gitrepo = Repo.clone_from(url, uuid4(), multi_options=["--recurse-submodules"])
             gitrepo.git.checkout(git_ref)
     else:
-        gitrepo = Repo.clone_from(url, uuid4(), depth=1)
+        gitrepo = Repo.clone_from(url, uuid4(), depth=1, multi_options=["--recurse-submodules"])
     commit_sha = gitrepo.head.commit.hexsha
     metadata, artifact_path = sync_collection(gitrepo.working_dir, ".")
     if not metadata_only:
@@ -137,44 +140,6 @@ async def declarative_content_from_git_repo(remote, url, git_ref=None, metadata_
     return d_content
 
 
-def update_collection_remote(remote_pk, *args, **kwargs):
-    """
-    Update a CollectionRemote.
-
-    Update `AnsibleRepository.last_synced_metadata_time` when URL or requirements file are updated.
-
-    Args:
-        remote_pk (str): the id of the CollectionRemote
-        data (dict): dictionary whose keys represent the fields of the model and their corresponding
-            values.
-        partial (bool): When true, only the fields specified in the data dictionary are updated.
-            When false, any fields missing from the data dictionary are assumed to be None and
-            their values are updated as such.
-
-    Raises:
-        :class:`rest_framework.exceptions.ValidationError`: When serializer instance can't be saved
-            due to validation error. This theoretically should never occur since validation is
-            performed before the task is dispatched.
-    """
-    data = kwargs.pop("data", None)
-    partial = kwargs.pop("partial", False)
-    with transaction.atomic():
-        if "url" in data or "requirements_file" in data:
-            repos = list(
-                AnsibleRepository.objects.filter(
-                    remote_id=remote_pk, last_synced_metadata_time__isnull=False
-                ).all()
-            )
-            for repo in repos:
-                repo.last_synced_metadata_time = None
-            AnsibleRepository.objects.bulk_update(repos, ["last_synced_metadata_time"])
-
-        instance = CollectionRemote.objects.get(pk=remote_pk)
-        serializer = CollectionRemoteSerializer(instance, data=data, partial=partial)
-        serializer.is_valid(raise_exception=True)
-        serializer.save()
-
-
 def sync(remote_pk, repository_pk, mirror, optimize):
     """
     Sync Collections with ``remote_pk``, and save a new RepositoryVersion for ``repository_pk``.
@@ -217,9 +182,8 @@ def sync(remote_pk, repository_pk, mirror, optimize):
     if not repo_version:
         return
 
-    if first_stage.last_synced_metadata_time:
-        repository.last_synced_metadata_time = first_stage.last_synced_metadata_time
-        repository.save()
+    repository.last_synced_metadata_time = first_stage.last_synced_metadata_time
+    repository.save(update_fields=["last_synced_metadata_time"])
 
     to_undeprecate = Q()
     undeprecated = deprecation_before_sync.difference(first_stage.deprecation_after_sync)
@@ -394,6 +358,57 @@ def create_collection_from_importer(importer_result, metadata_only=False):
     return collection_version
 
 
+def rebuild_repository_collection_versions_metadata(
+    repository_version_pk, namespace=None, name=None, version=None
+):
+    """Rebuild metadata for all collection versions in a repo."""
+    repov = RepositoryVersion.objects.get(pk=repository_version_pk)
+
+    qs = None
+    if namespace or name or version:
+        qkwargs = {}
+        if namespace:
+            qkwargs["namespace"] = namespace
+        if name:
+            qkwargs["name"] = name
+        if version:
+            qkwargs["version"] = version
+        qs = CollectionVersion.objects.filter(**qkwargs)
+
+    qs = repov.get_content(content_qs=qs)
+    with ProgressReport(
+        message=_("Rebuild collection version metadata (total)"),
+        code="rebuild_metadata.total",
+        total=qs.count(),
+    ) as ptotal, ProgressReport(
+        message=_("Rebuild collection version metadata (failed)"), code="rebuild_metadata.failed"
+    ) as pfailed:
+        for cv in qs:
+            try:
+                _rebuild_collection_version_meta(cv)
+            except Exception as e:
+                pfailed.increment()
+                log.exception(e)
+            ptotal.increment()
+
+
+def _rebuild_collection_version_meta(collection_version):
+    """Rebuild metadata for a single collection version."""
+    # where is the artifact?
+    artifact = collection_version._artifacts.first()
+
+    # call the importer to re-generate meta
+    importer_result = process_collection(
+        artifact.file, filename=artifact.file.name, file_url=artifact.file.url, logger=None
+    )
+
+    # set the new info and save
+    collection_version.requires_ansible = importer_result["requires_ansible"]
+    collection_version.docs_blob = importer_result["docs_blob"]
+    collection_version.contents = importer_result["contents"]
+    collection_version.save()
+
+
 def _get_backend_storage_url(artifact_file):
     """Get artifact url from pulp backend storage."""
     if (
@@ -492,7 +507,6 @@ class CollectionSyncFirstStage(Stage):
         super().__init__()
         self.remote = remote
         self.repository = repository
-        self.is_repo_remote = is_repo_remote
         self.deprecation_before_sync = deprecation_before_sync
         self.deprecation_after_sync = set()
         self.collection_info = parse_collections_requirements_file(remote.requirements_file)
@@ -502,13 +516,14 @@ class CollectionSyncFirstStage(Stage):
         self.already_synced = set()
         self._unpaginated_collection_metadata = None
         self._unpaginated_collection_version_metadata = None
+        self.optimize = optimize
         self.last_synced_metadata_time = None
 
         # Interpret download policy
         self.deferred_download = self.remote.policy != Remote.IMMEDIATE
 
         # Check if we should sync
-        self.should_sync = not optimize or asyncio.get_event_loop().run_until_complete(
+        self.should_sync = not is_repo_remote or asyncio.get_event_loop().run_until_complete(
             self._should_we_sync()
         )
 
@@ -628,11 +643,11 @@ class CollectionSyncFirstStage(Stage):
         if signatures:
             collection_version = await d_content.resolution()
             for signature in signatures:
-                sig = signature["signature"].encode("utf8")
+                sig = signature["signature"]
                 cv_signature = CollectionVersionSignature(
                     signed_collection=collection_version,
                     data=sig,
-                    digest=hashlib.sha256(sig).hexdigest(),
+                    digest=hashlib.sha256(sig.encode("utf-8")).hexdigest(),
                     pubkey_fingerprint=signature["pubkey_fingerprint"],
                 )
                 await self.put(DeclarativeContent(content=cv_signature))
@@ -899,9 +914,6 @@ class CollectionSyncFirstStage(Stage):
         """Check last synced metadata time."""
         msg = _("no_change: Checking if remote changed since last sync.")
         async with ProgressReport(message=msg, code="sync.no_change") as noop:
-            if not self.is_repo_remote:
-                return True
-
             root, api_version = await self._get_root_api(self.remote.url)
             if api_version == 3:
                 downloader = self.remote.get_downloader(
@@ -917,6 +929,9 @@ class CollectionSyncFirstStage(Stage):
                 except KeyError:
                     return True
 
+                if not self.optimize:
+                    return True
+
                 sources = set()
                 if self.collection_info:
                     sources = {r.source for r in self.collection_info if r.source}
